---
-- VehicleWorkshop
--
-- Author: Rockstar
-- Version: 1.0.0.0
-- Date: 29.10.2020
--

VehicleWorkshop = {}
VehicleWorkshop.MAX_ACTIVATABLE_DISTANCE = 3
VehicleWorkshop.OWN_WORKSHOP = false

function VehicleWorkshop:new()
	self.g_directSellDialogOnOpenCallbackOld = g_directSellDialog.elements[1].onOpenCallback
	self.workshopText = g_i18n:getText("action_configSellSpecificVehicle")
	self.isActivatableAdded = false

	return self
end

function VehicleWorkshop:loadMap(filename)
	if g_seasons ~= nil then
		self.g_directSellDialogOnOpenCallbackNew = g_directSellDialog.elements[1].onOpenCallback
	else
		self.g_directSellDialogOnOpenCallbackOld = nil
	end

	if _G["FS19_workshopTabber"] ~= nil and g_modIsLoaded["FS19_workshopTabber"] then
		g_directSellDialog.keyEvent = Utils.overwrittenFunction(g_directSellDialog.keyEvent, function(object, superFunc, unicode, sym, modifier, isDown)
			if object.owner ~= self then
				superFunc(object, unicode, sym, modifier, isDown)
			end
		end)
	end
end

function VehicleWorkshop:deleteMap()
	g_currentMission:removeActivatableObject(self)
end

function VehicleWorkshop:update(dt)
	if g_currentMission:getIsClient() then
		local player = g_currentMission.player
		if player ~= nil then
			if player.isControlled then
				local x, y, z = localToWorld(player.cameraNode, 0, 0, 1)
				local dx, dy, dz = localDirectionToWorld(player.cameraNode, 0, 0, -1)
				self.lastFoundVehicle = nil
				raycastAll(x, y, z, dx, dy, dz, "vehicleRaycastCallback", VehicleWorkshop.MAX_ACTIVATABLE_DISTANCE, self)
				self.isVehicleInRange = self.lastFoundVehicle ~= nil
			end
		end

		self:addActivatableObject(self.isVehicleInRange)
	end
end

function VehicleWorkshop:vehicleRaycastCallback(hitObjectId, x, y, z, distance)
	if hitObjectId ~= g_currentMission.terrainDetailId then
		local object = g_currentMission:getNodeObject(hitObjectId)

		if object ~= nil then
			if object.isa ~= nil and object:isa(Vehicle) then
				if entityExists(object.rootNode) then
					if getRigidBodyType(object.rootNode) == "Dynamic" then
						self.lastFoundVehicle = object
						return false
					end
				end
			end
		end
	end
	return true
end

function VehicleWorkshop:addActivatableObject(vehicleInRange)
	if vehicleInRange then
		if not self.isActivatableAdded then
			self:updateActivateText()
			g_currentMission:addActivatableObject(self)
			self.isActivatableAdded = true
		end
		if self.previousVehicle == nil then
			self.previousVehicle = self.lastFoundVehicle
		elseif self.previousVehicle ~= self.lastFoundVehicle then
			self:updateActivateText()
			self.previousVehicle = self.lastFoundVehicle
		end
	elseif self.isActivatableAdded then
		g_currentMission:removeActivatableObject(self)
		self.previousVehicle = nil
		self.isActivatableAdded = false
	end
end

function VehicleWorkshop:getIsActivatable()
	return g_currentMission.controlPlayer and not g_currentMission.player.isCarryingObject and (self.lastFoundVehicle:getOwnerFarmId() == 0 or g_currentMission:getFarmId() == self.lastFoundVehicle:getOwnerFarmId())
end

function VehicleWorkshop:drawActivate()
end

function VehicleWorkshop:onActivateObject()
	self:showSellDialog(self.lastFoundVehicle, self, VehicleWorkshop.OWN_WORKSHOP)
end

function VehicleWorkshop:showSellDialog(vehicle, owner, ownWorkshop)
	local replaceFunc = g_seasons ~= nil

	if replaceFunc then
		g_directSellDialog.elements[1].onOpenCallback = self.g_directSellDialogOnOpenCallbackOld
	end

	g_gui:showDirectSellDialog({
		vehicle = vehicle,
		owner = owner,
		ownWorkshop = ownWorkshop
	})

	if replaceFunc then
		g_directSellDialog.elements[1].onOpenCallback = self.g_directSellDialogOnOpenCallbackNew
	end
end

function VehicleWorkshop:shouldRemoveActivatable()
	return false
end

function VehicleWorkshop:updateActivateText()
	self.activateText = string.format("%s: %s", self.workshopText, self.lastFoundVehicle:getFullName())
end

addModEventListener(VehicleWorkshop:new())